@@ -35,7 +35,7 @@
 #define _XF86DRM_H_
 
 #include <stdarg.h>
-#if defined(__NetBSD__) && __NetBSD_Version__ >= 499003700
+#if defined(__NetBSD__)
 #include <sys/atomic.h>
 #else
 #include <sys/types.h>
@@ -309,9 +309,11 @@ typedef struct _drmSetVersion {
 #define DRM_LOCK_HELD  0x80000000U /**< Hardware lock is held */
 #define DRM_LOCK_CONT  0x40000000U /**< Hardware lock is contended */
 
-#if defined(__NetBSD__) && __NetBSD_Version__ >= 499003700
+#if defined(__NetBSD__)
+
 #define DRM_CAS(lock, old, new, __ret)	\
  (__ret = atomic_cas_uint(&__drm_dummy_lock(lock), (old), (new)) != (old));
+
 #elif defined(__GNUC__) && (__GNUC__ >= 2)
 # if defined(__i386) || defined(__AMD64__) || defined(__x86_64__) || defined(__amd64__)
 				/* Reflect changes here to drmP.h */
@@ -434,7 +436,7 @@ do {	register unsigned int __old __asm("o0");		\
 #define DRM_CAS(lock,old,new,ret) do { ret=1; } while (0) /* FAST LOCK FAILS */
 #endif
 
-#if defined(__alpha__) || defined(__powerpc__)
+#if defined(__alpha__) || defined(__powerpc__) && !defined(__NetBSD__)
 #define DRM_CAS_RESULT(_result)		int _result
 #else
 #define DRM_CAS_RESULT(_result)		char _result
@@ -665,4 +667,6 @@ extern int  drmSLLookupNeighbors(void *l, unsigned long key,
 extern int drmOpenOnce(void *unused, const char *BusID, int *newlyopened);
 extern void drmCloseOnce(int fd);
 
+#include "xf86mm.h"
+
 #endif
